@@ -71,11 +71,15 @@ class Cell:
         self.replace_missing_ref = replace_missing_ref
         if reference is not None:
             self.range = Ranges().push(reference, context=context)
-            self.output = self.range.ranges[0]['name']
-        self.tokens, self.builder, self.value = (), None, sh.EMPTY
+            r = self.range.ranges[0]
+            context = sh.combine_dicts(context or {}, base={
+                'cr': r['r1'], 'cc': r['n1']
+            })
+            self.output = r['name']
+        self.builder, self.value = None, sh.EMPTY
         prs = self.parser
         if check_formula and isinstance(value, str) and prs.is_formula(value):
-            self.tokens, self.builder = prs.ast(value, context=context)
+            self.builder = prs.ast(value, context=context)[1]
         elif value is not None:
             self.value = value
 
@@ -86,12 +90,13 @@ class Cell:
         return self.output
 
     def compile(self, references=None, context=None):
-        if self.builder:
+        if not self.func and self.builder:
             func = self.builder.compile(
                 references=references, context=context, **{CELL: self.range}
             )
             self.func = wrap_cell_func(func, self._args)
             self.update_inputs(references=references)
+            self.builder = None
         return self
 
     def _missing_ref(self, inp, k):
@@ -103,8 +108,6 @@ class Cell:
         sh.get_nested_dicts(inp, i, default=list).append(k)
 
     def update_inputs(self, references=None):
-        if not self.builder:
-            return
         self.inputs = inp = collections.OrderedDict()
         references, get = references or set(), sh.get_nested_dicts
         for k, rng in self.func.inputs.items():
@@ -185,8 +188,6 @@ class Ref(Cell):
         super(Ref, self).compile(references=references, context=context)
         if self.inputs:
             self.func.dsp.nodes[self.func.outputs[0]].pop('filters', None)
-        elif self.value is not sh.EMPTY:
-            self.value, self.func = self.func(), None
         return self
 
 
@@ -272,7 +273,10 @@ class RangesAssembler:
                 if isinstance(v, dict):
                     v = ists[n] = _get_indices_intersection(base, v)
                 i, j = v
-                out[i, j] = sol[n].value
+                v = sol[n]
+                if isinstance(sol[n], Ranges):
+                    v = v.value
+                out[i, j] = v
         else:
             out = np.empty(_shape(**base), object)
         for c, ind in zip(cells, self.inputs.values()):
