@@ -55,6 +55,14 @@ def _get_name(name, names):
     return name
 
 
+def _encode_path(path):
+    return path.replace('\\', '/')
+
+
+def _decode_path(path):
+    return path.replace('/', osp.sep)
+
+
 class ExcelModel:
     compile_class = sh.DispatchPipe
 
@@ -114,31 +122,39 @@ class ExcelModel:
         formula_references = self.formula_references(context)
         formula_ranges = self.formula_ranges(context)
         external_links = self.external_links(context)
-
+        ctx = {'external_links': external_links}
+        ctx.update(context)
+        cells = []
         for row in worksheet.iter_rows():
             for c in row:
                 if hasattr(c, 'value'):
-                    self.add_cell(
-                        c, context, references=references,
-                        formula_references=formula_references,
-                        formula_ranges=formula_ranges,
-                        external_links=external_links
-                    )
+                    cells.append(self.compile_cell(
+                        c, ctx, references, formula_references
+                    ))
+        for cell in cells:
+            # noinspection PyTypeChecker
+            self.add_cell(sh.await_result(cell), ctx, formula_ranges)
         return self
 
     def add_book(self, book=None, context=None, data_only=False):
         ctx = (context or {}).copy()
         are_in, get_in = sh.are_in_nested_dicts, sh.get_nested_dicts
         if isinstance(book, str):
-            ctx['directory'], ctx['filename'] = osp.split(book)
+            book = _encode_path(book).split('/')
+            ctx['directory'], ctx['filename'] = '/'.join(book[:-1]), book[-1]
         if self.basedir is None:
-            self.basedir = osp.abspath(ctx['directory'] or '.')
+            directory = _encode_path(ctx.get('directory') or '.')
+            self.basedir = osp.abspath(_decode_path(directory))
+            ctx['directory'] = ''
         if ctx['directory']:
-            ctx['directory'] = osp.relpath(ctx['directory'], self.basedir)
+            ctx['directory'] = _encode_path(osp.relpath(
+                osp.join(self.basedir, _decode_path(ctx['directory'])),
+                self.basedir
+            ))
         if ctx['directory'] == '.':
             ctx['directory'] = ''
-        fpath = osp.join(ctx['directory'], ctx['filename'])
-        ctx['excel'] = fpath.upper()
+        fpath = osp.join(_decode_path(ctx['directory']), ctx['filename'])
+        ctx['excel'] = _encode_path(fpath).upper()
         data = get_in(self.books, ctx['excel'])
         book = data.get(BOOK)
         if not book:
@@ -148,14 +164,18 @@ class ExcelModel:
             )
 
         if 'external_links' not in data:
-            fdir = osp.join(self.basedir, ctx['directory'])
+            fdir = osp.join(self.basedir, _decode_path(ctx['directory']))
             data['external_links'] = {
                 str(i + 1): osp.split(osp.relpath(osp.realpath(osp.join(
-                    fdir, el.file_link.Target
+                    fdir, _decode_path(el.file_link.Target)
                 )), self.basedir))
                 for i, el in enumerate(book._external_links)
                 if el.file_link.Target.endswith('.xlsx')
             }
+            data['external_links'] = {
+                k: (_encode_path(d), f)
+                for k, (d, f) in data['external_links'].items()
+            }
 
         if 'references' not in data:
             data['references'] = self.add_references(
@@ -210,37 +230,28 @@ class ExcelModel:
     def external_links(self, ctx):
         return sh.get_nested_dicts(self.books, ctx['excel'], 'external_links')
 
-    def add_cell(self, cell, context, references=None, formula_references=None,
-                 formula_ranges=None, external_links=None):
-        if formula_references is None:
-            formula_references = self.formula_references(context)
+    @staticmethod
+    def _compile_cell(crd, val, context, check_formula, references):
+        cell = Cell(crd, val, context=context, check_formula=check_formula)
+        cell.compile(references=references, context=context)
+        return cell
 
-        if formula_ranges is None:
-            formula_ranges = self.formula_ranges(context)
-
-        if references is None:
-            references = self.references
-
-        if external_links is None:
-            external_links = self.external_links(context)
-
-        ctx = {'external_links': external_links}
-        ctx.update(context)
+    def compile_cell(self, cell, context, references, formula_references):
         crd = cell.coordinate
         crd = formula_references.get(crd, crd)
         val = cell.value
         val = cell.data_type == 'f' and val[:2] == '==' and val[1:] or val
         check_formula = cell.data_type != 's'
-        cell = Cell(crd, val, context=ctx, check_formula=check_formula).compile(
-            references=references, context=ctx
-        )
+        return self._compile_cell(crd, val, context, check_formula, references)
+
+    def add_cell(self, cell, context, formula_ranges):
         if cell.output in self.cells:
             return
         if cell.value is not sh.EMPTY:
             if any(not (cell.range - rng).ranges for rng in formula_ranges):
                 return
 
-        if cell.add(self.dsp, context=ctx):
+        if cell.add(self.dsp, context=context):
             self.cells[cell.output] = cell
             return cell
 
@@ -264,8 +275,10 @@ class ExcelModel:
                 log.warning('Missing Reference `{}`!'.format(n_id))
                 Ref(n_id, '=#REF!').compile().add(self.dsp)
                 continue
-            book = osp.join(rng.get('directory', ''),
-                            rng.get('filename', rng.get('excel_id', '')))
+            book = _encode_path(osp.join(
+                _decode_path(rng.get('directory', '')),
+                _decode_path(rng.get('filename', rng.get('excel_id', '')))
+            ))
 
             try:
                 context = self.add_book(book)[1]
@@ -273,7 +286,7 @@ class ExcelModel:
             except Exception as ex:  # Missing excel file or sheet.
                 log.warning('Error in loading `{}`:\n{}'.format(n_id, ex))
                 Cell(n_id, '=#REF!').compile().add(self.dsp)
-                self.books.pop(book)
+                self.books.pop(book, None)
                 continue
 
             references = self.references
@@ -291,19 +304,22 @@ class ExcelModel:
                 int(rng['r1']), min(int(rng['r2']), max_row),
                 rng['n1'], min(rng['n2'], max_column)
             )
+            ctx = {'external_links': external_links}
+            ctx.update(context)
+            cells = []
             for c in flatten(it, None):
                 n = _name % c.coordinate
                 if n in self.cells:
                     continue
                 elif hasattr(c, 'value'):
-                    cell = self.add_cell(
-                        c, context, references=references,
-                        formula_references=formula_references,
-                        formula_ranges=formula_ranges,
-                        external_links=external_links
-                    )
-                    if cell:
-                        stack.extend(cell.inputs or ())
+                    cells.append(self.compile_cell(
+                        c, ctx, references, formula_references
+                    ))
+            for cell in cells:
+                # noinspection PyTypeChecker
+                cell = self.add_cell(sh.await_result(cell), ctx, formula_ranges)
+                if cell:
+                    stack.extend(cell.inputs or ())
         return self
 
     def _assemble_ranges(self, cells, nodes=None, compact=1):
@@ -317,8 +333,6 @@ class ExcelModel:
         )
         ranges = []
         for n_id in it:
-            if isinstance(n_id, sh.Token):
-                continue
             try:
                 ra = RangesAssembler(n_id, compact=compact)
             except ValueError:
@@ -434,7 +448,9 @@ class ExcelModel:
                     rng = r.ranges[0]
                 except ValueError:  # Reference.
                     rng = {'sheet': ''}
-            fpath = osp.join(rng.get('directory', ''), rng.get('filename', ''))
+            fpath = _encode_path(osp.join(
+                _decode_path(rng.get('directory', '')), rng.get('filename', '')
+            ))
             fpath, sheet_name = _get_name(fpath, books), rng.get('sheet')
             if not (fpath and sheet_name):
                 log.info('Node `%s` cannot be saved '
@@ -455,7 +471,8 @@ class ExcelModel:
             if sheet_name not in sheet_names:
                 book.create_sheet(sheet_name)
             sheet = book[sheet_name]
-
+            rng['c1'] = rng['c1'] or 'A'
+            rng['r1'] = int(rng['r1']) or 1
             ref = '{c1}{r1}:{c2}{r2}'.format(**rng)
             for c, v in zip(flatten(sheet[ref], None), flatten(r.value, None)):
                 try:
@@ -473,7 +490,7 @@ class ExcelModel:
         if dirpath:
             os.makedirs(dirpath, exist_ok=True)
             for fpath, d in books.items():
-                d[BOOK].save(osp.join(dirpath, fpath))
+                d[BOOK].save(osp.join(dirpath, _decode_path(fpath)))
         return books
 
     def compile(self, inputs, outputs):
