@@ -23,6 +23,10 @@ def inp_ranges(*rng):
 @ddt.ddt
 class TestCell(unittest.TestCase):
     @ddt.idata([
+        ('A1', '=ROMAN(0)', {}, '<Ranges>(A1)=[[\'\']]'),
+        ('A1', '=ROMAN(1)', {}, '<Ranges>(A1)=[[\'I\']]'),
+        ('A1', '=ROMAN(4000)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=ROMAN(1, 5)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
         ('A1', '=ADDRESS(2,3,2,FALSE)', {}, '<Ranges>(A1)=[[\'R2C[3]\']]'),
         ('A1', '=ADDRESS(2,3,2)', {}, '<Ranges>(A1)=[[\'C$2\']]'),
         ('A1', '=ADDRESS(2,3)', {}, '<Ranges>(A1)=[[\'$C$2\']]'),
@@ -60,6 +64,12 @@ class TestCell(unittest.TestCase):
          '<Ranges>(A1)=[[152]]'),
         ('A1', '=DATEDIF("01/09/01","06/02/08","YD")', {},
          '<Ranges>(A1)=[[144]]'),
+        ('A1', '=DATEDIF("12/31/2001","12/29/2008","MD")', {},
+         '<Ranges>(A1)=[[28]]'),
+        ('A1', '=DATEDIF("12/20/2001","12/19/2008","MD")', {},
+         '<Ranges>(A1)=[[29]]'),
+        ('A1', '=DATEDIF("12/29/2001","12/29/2008","MD")', {},
+         '<Ranges>(A1)=[[0]]'),
         ('A1', '=DATEDIF("01/31/2001","01/29/2008","YD")', {},
          '<Ranges>(A1)=[[363]]'),
         ('A1', '=DATEDIF("01/01/2001","02/29/2008","YD")', {},
@@ -98,6 +108,12 @@ class TestCell(unittest.TestCase):
         ('A1', '=DATEDIF(222,222290,"D")', {}, '<Ranges>(A1)=[[222068]]'),
         ('A1', '=DATEDIF(222,222289,"D")', {}, '<Ranges>(A1)=[[222067]]'),
         ('A1', '=DATEDIF(222,222222,"D")', {}, '<Ranges>(A1)=[[222000]]'),
+        ('A1', '=EDATE(1, TRUE)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=EDATE(1, "DSD")', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=WEEKDAY(-1, 21)', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=DAY(1)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(-1, 21)', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=WEEKNUM(1, 22)', {}, '<Ranges>(A1)=[[#NUM!]]'),
         ('A1', '=WEEKNUM(0, 21)', {}, '<Ranges>(A1)=[[52]]'),
         ('A1', '=WEEKNUM(1, 21)', {}, '<Ranges>(A1)=[[52]]'),
         ('A1', '=WEEKNUM(2, 21)', {}, '<Ranges>(A1)=[[1]]'),
@@ -353,6 +369,16 @@ class TestCell(unittest.TestCase):
          {'A2:F2': [[60, "29/02/1900", sh.EMPTY, 0, "*", "AUG-98"]],
           'A3:F3': [[1, 3, 11, 7, 9, 13]]},
          '<Ranges>(A1:C1)=[[4.0 4.0 3.0]]'),
+        ('A1', '=YEARFRAC("31/8/1987","29/8/1986",0)', {},
+         '<Ranges>(A1)=[[1.0055555555555555]]'),
+        ('A1', '=YEARFRAC("31/8/1987","29/8/1986",4)', {},
+         '<Ranges>(A1)=[[1.0027777777777778]]'),
+        ('A1', '=YEARFRAC("30/8/1987","29/8/1986",0)', {},
+         '<Ranges>(A1)=[[1.0027777777777778]]'),
+        ('A1', '=YEARFRAC("31/8/1987","30/8/1986",0)', {},
+         '<Ranges>(A1)=[[1.0]]'),
+        ('A1', '=YEARFRAC("30/8/1987","31/8/1986",0)', {},
+         '<Ranges>(A1)=[[1.0]]'),
         ('A1', '=YEARFRAC(0,345,TRUE)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
         ('A1', '=YEARFRAC("26/8/1987 05:00 AM",345,4)', {},
          '<Ranges>(A1)=[[86.71111111111111]]'),
@@ -439,7 +465,10 @@ class TestCell(unittest.TestCase):
         ('A1', '=OR(TRUE,#REF!,"0")', {}, '<Ranges>(A1)=[[#REF!]]'),
         ('A1', '=OR(FALSE,"0",#REF!)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
         ('A1', '=INDEX({2,3;4,5},FALSE,"0")', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=INDEX({2,3;4,5},0,0)', {}, '<Ranges>(A1)=[[2]]'),
         ('A1', '=INDEX({2,3;4,5}, -1)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=INDEX({2,3;4,5}, 0, 0, -1)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=INDEX({2,3;4,5}, 0, -1)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
         ('A1', '=INDEX(B1:C1, 1, 1)', {'B1:C1': [[sh.EMPTY, 2]]},
          '<Ranges>(A1)=[[0]]'),
         ('A1', '=INDEX(B1:C2, -1)', {'B1:C2': [[1, 2], [3, 4]]},
@@ -466,6 +495,7 @@ class TestCell(unittest.TestCase):
         ('A1', '=LOOKUP(2,{-1.1,2.1,3.1,4.1})', {}, '<Ranges>(A1)=[[-1.1]]'),
         ('A1', '=LOOKUP(3,{-1.1,2.1,3.1,4.1})', {}, '<Ranges>(A1)=[[2.1]]'),
         ('A1', '=SWITCH(TRUE,1,0,,,TRUE,1,7)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=SWITCH(TRUE,1,0,#REF!,3,TRUE,1,7)', {}, '<Ranges>(A1)=[[#REF!]]'),
         ('A1:D1', '=SWITCH({0,1,TRUE},1,0,,,TRUE,1,7)', {},
          '<Ranges>(A1:D1)=[[0 0 1 #N/A]]'),
         ('A1', '=SWITCH(1,2,0,1,4,,4,5)', {}, '<Ranges>(A1)=[[4]]'),
@@ -507,6 +537,7 @@ class TestCell(unittest.TestCase):
          '<Ranges>(A11)=[[54]]'),
         ('A11', '=ROW(L45)', inp_ranges('L45'), '<Ranges>(A11)=[[45]]'),
         ('A11', '=ROW()', {}, '<Ranges>(A11)=[[11]]'),
+        ('A1', '=REF()', {}, "<Ranges>(A1)=[[#NAME?]]"),
         ('A1', '=REF', {}, "<Ranges>(A1)=[[#REF!]]"),
         ('A1', '=(-INT(2))', {}, '<Ranges>(A1)=[[-2.0]]'),
         ('A1', '=(1+1)+(1+1)', {}, '<Ranges>(A1)=[[4.0]]'),
