@@ -18,12 +18,14 @@ import schedula as sh
 from formulas.excel import ExcelModel, BOOK, ERR_CIRCULAR
 from formulas.excel.xlreader import load_workbook
 from formulas.functions import is_number
+from formulas.ranges import Ranges
 
 EXTRAS = os.environ.get('EXTRAS', 'all')
 
 mydir = osp.join(osp.dirname(__file__), 'test_files')
 _filename = 'test.xlsx'
 _filename_compile = 'excel.xlsx'
+_filename_full_range = 'full-range.xlsx'
 _link_filename = 'test_link.xlsx'
 _filename_circular = 'circular.xlsx'
 
@@ -45,7 +47,7 @@ def _res2books(res):
 
 def _file2books(*fpaths):
     d = osp.dirname(fpaths[0])
-    return {osp.relpath(fp, d).upper(): _book2dict(
+    return {osp.relpath(fp, d).upper().replace('\\', '/'): _book2dict(
         load_workbook(fp, data_only=True)
     ) for fp in fpaths}
 
@@ -57,6 +59,7 @@ class TestExcelModel(unittest.TestCase):
         self.link_filename = osp.join(mydir, _link_filename)
         self.filename_compile = osp.join(mydir, _filename_compile)
         self.filename_circular = osp.join(mydir, _filename_circular)
+        self.filename_full_range = osp.join(mydir, _filename_full_range)
 
         self.results = _file2books(self.filename, self.link_filename)
         sh.get_nested_dicts(self.results, 'EXTRA.XLSX', 'EXTRA').update({
@@ -66,6 +69,13 @@ class TestExcelModel(unittest.TestCase):
             load_workbook(self.filename_compile, data_only=True)
         )['DATA']
         self.results_circular = _file2books(self.filename_circular)
+        self.results_full_range = {
+            'TEST_FILES/%s' % k: v
+            for k, v in _file2books(self.filename_full_range).items()
+        }
+        sh.get_nested_dicts(
+            self.results_full_range, 'TEST_FILES/FULL-RANGE.XLSX', 'DATA'
+        ).update({'A6': 5, 'A7': 1, 'B2': 19, 'B5': 25})
         self.maxDiff = None
 
     def _compare(self, books, results):
@@ -84,7 +94,10 @@ class TestExcelModel(unittest.TestCase):
             except AssertionError as ex:
                 errors.append(str(ex))
         self.assertFalse(
-            bool(errors), 'Errors(%d):\n%s' % (len(errors), '\n'.join(errors))
+            bool(errors),
+            'Errors({}):\n{}\n\nCalculated:\n{}\n\nTarget:\n{}'.format(
+                len(errors), '\n'.join(errors), books, results
+            )
         )
         return len(it)
 
@@ -129,7 +142,8 @@ class TestExcelModel(unittest.TestCase):
                 print(msg % (_msg, time.time() - s, _msg))
                 s = time.time()
 
-                n_test += self._compare(_res2books(xl_mdl.write()), self.results)
+                n_test += self._compare(_res2books(xl_mdl.write()),
+                                        self.results)
 
                 msg = '%sCompared fresh written results in %.2fs.'
                 print(msg % (_msg, time.time() - s))
@@ -233,5 +247,33 @@ class TestExcelModel(unittest.TestCase):
 
         self._compare(books, self.results_circular)
 
+    def test_excel_model_full_range(self):
+        fname = osp.basename(self.filename_full_range)
+        xl_model = ExcelModel()
+        xl_model.basedir = osp.dirname(__file__)
+        xl_model.complete([
+            f"'test_files/[{fname}]DATA'!B5"
+        ]).finish(complete=False)
+        sheet_name = f'test_files/[{fname}]DATA'
+        xl_model.calculate({
+            f"'{sheet_name}'!A6": 5,
+            f"'{sheet_name}'!A7": Ranges().push(f"'{sheet_name}'!A7", 1)
+        })
+        books = {
+            k: _book2dict(v[BOOK]) for k, v in
+            xl_model.write(xl_model.books).items()
+        }
+
+        self._compare(books, self.results_full_range)
+
+    def test_excel_from_dict(self):
+        xl_model = ExcelModel().from_dict({
+            'A1': 1, 'B2': '=R[-1]C[-1]', 'A': 2, 'B': '=2'
+        }).finish()
+        self.assertEqual({
+            k: v.value if isinstance(v, Ranges) else v
+            for k, v in xl_model.calculate().items()
+        }, {'A1': 1, 'B2': 1, 'A': 2, 'B': 2})
+
     def tearDown(self) -> None:
         shutil.rmtree(osp.join(mydir, 'tmp'), ignore_errors=True)
