@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2021 European Commission (JRC);
+# Copyright 2016-2022 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -14,8 +14,8 @@ EXTRAS = os.environ.get('EXTRAS', 'all')
 
 
 @unittest.skipIf(EXTRAS not in ('all',), 'Not for extra %s.' % EXTRAS)
-class TestDoctest(unittest.TestCase):
-    def runTest(self):
+class TestReadme(unittest.TestCase):
+    def test_readme(self):
         failure_count, test_count = doctest.testfile(
             '../README.rst', optionflags=doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
         )
